<?php

/**
 * @name Knowtify Class
 */
class Knowtify {

    var $apiKey;
    var $plugin_url;
    var $apiURL = "http://www.knowtify.io/api/v1";

    /**
     * Fire up the class.
     */
    function __construct() {
        $this->apiKey = get_option("Knowtify_APIKEY");
        $this->plugin_url = plugin_dir_url(__FILE__);
        if (!get_option("KT_cb_text")) {
            update_option("KT_cb_text", "Subscribe to our email digest");
        }
        if (!get_option("KT_digest_heading")) {
            /**
             * if KT_digest_heading isn't set then add defaults and update API
             */
            update_option("KT_show_cb", 1);
            update_option("KT_auto_cb", 1);
            update_option("KT_show_signup_form", 1);
            update_option("KT_signup_label", "Sign up to receive our weekly digest email!");
            update_option("KT_signup_thanks", "Thanks for signing up!");
            update_option("KT_signup_button_text", "Sign Up!");
            update_option("KT_digest_from", get_bloginfo('name'));
            update_option("KT_digest_subject", get_bloginfo('name') . " Weekly Digest");
            update_option("KT_digest_from", get_bloginfo('name'));
            update_option("KT_digest_from_email", get_bloginfo('admin_email'));
            update_option("KT_digest_header_image", "https://s3.amazonaws.com/knowtify_web/_backgrounds/coffee.jpg");
            update_option("KT_digest_heading", "Fresh off the press!");
            #$this->update_digest();
        }
        $this->actions();
    }

    /**
     * WordPress Actions/Filters
     */
    function actions() {

        if ($this->apiKey == '') { // API key is not saved in WP Options.
            add_action('admin_notices', array($this, 'notice'));
        }
        if (is_admin()) {
            add_action('admin_menu', array($this, 'setupPage'));
            add_action('admin_init', array($this, 'register_mysettings'));
            add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
            add_action('wp_ajax_new_signup', array($this, 'signup_ajax_response'));
        }
        add_action('comment_form', array($this, 'comment_form'));
        add_action('comment_post', array($this, 'comment_subscribe'));

        add_filter('the_content', array($this, 'signup_form'));

        add_action('publish_post', array($this, 'add_post'));

        /* Reader signup */
        add_action('wp_ajax_nopriv_new_reader', array($this, 'reader_ajax_response'));

        /* CSS and JS are needed on the post page for the signup form in addition to the admin page */
        add_action('wp_head', array($this, 'add_css'));
        add_action('wp_footer', array($this, 'add_js'));
    }

    /* Add CSS */
    function add_css(){
        wp_enqueue_style('knowtify-css', $this->plugin_url . "assets/css/knowtify.css");
    }

    /* Add JS */
    function add_js(){
        wp_enqueue_script('knowtify-admin', plugins_url('/assets/js/knowtify.js', __FILE__), array('jquery'));

        // in javascript, object properties are accessed as ajax_object.ajax_url, ajax_object.we_value
        wp_localize_script('knowtify-admin', 'ajax_object', array('ajax_url' => admin_url('admin-ajax.php')));
    }

    /**
     * Display a notice to users who don't have an API key.
     */
    function notice() {
        if (isset($_GET['page'])) {
            if ($_GET['page'] == 'knowtify') {
                return;
            }
        }
        $siteurl = get_option('siteurl');
        $siteurl.="/wp-admin/admin.php?page=knowtify";
        include plugin_dir_path(__FILE__) . "views/notice.php";
    }

    /**
     * Admin Menu Items
     */
    function setupPage() {
        $page = add_menu_page('Knowtify', 'Knowtify', 'manage_options', "knowtify", array($this, 'adminpage'), $this->plugin_url . "assets/images/icon.png");
        add_action('admin_print_styles-' . $page, array($this, "admin_style"));
    }

    /**
     * Admin Page Template
     */
    function adminpage($current = "account") {
        if (empty($this->apiKey)) {
            $tabs = array("signup" => "Create New Account");
        }else{
            $tabs = array("about" => "About", "digest" => "Digest Design", "general" => "Blog Settings", "account" => "Account");
        }
        if (isset($_GET['tab'])) {
            $tab = (string) $_GET['tab'];
            if (array_key_exists($tab, $tabs)) {  // Is this a valid tab?
                $current = $tab;

                if(isset($_GET['settings-updated']) && $tab == 'digest'){ // This is a hack
                    $this->update_digest();
                }
            } else {
                $current = "about";
            }
        } else {
            $current = "about";
        }


        include plugin_dir_path(__FILE__) . "views/adminpage.php";
        #if (!empty($this->apiKey)) { $this->add_contact(); }
        if (isset($_GET['newaccount'])) {
            # $this->create_account();
        }
    }

    /**
     * Register admin form options.
     */
    function register_mysettings() {
        register_setting('knowtify-group', 'Knowtify_APIKEY');
        register_setting('knowtify-group2', 'KT_show_cb');
        register_setting('knowtify-group2', 'KT_auto_cb');
        register_setting('knowtify-group2', 'KT_cb_text');
        register_setting('knowtify-group3', 'KT_digest_subject');
        register_setting('knowtify-group3', 'KT_digest_from');
        register_setting('knowtify-group3', 'KT_digest_from_email');
        register_setting('knowtify-group3', 'KT_digest_header_image');
        register_setting('knowtify-group3', 'KT_digest_heading');
        register_setting('knowtify-group3', 'KT_digest_intro');
        register_setting('knowtify-group3', 'KT_digest_address');
        register_setting('knowtify-group4', 'KT_show_signup_form');
        register_setting('knowtify-group4', 'KT_signup_label');
        register_setting('knowtify-group4', 'KT_signup_thanks');
        register_setting('knowtify-group4', 'KT_signup_button_text');
    }

    /**
     * Add admin CSS.
     */
    function admin_style() {
        wp_enqueue_style('knowtify-css', $this->plugin_url . "assets/css/knowtify.css");
    }

    /**
     * Add admin JS.
     */
    function admin_scripts($hook) {

        if ('index.php' != $hook) {
            // Only applies to dashboard panel
            // return;
        }

        wp_enqueue_script('knowtify-admin', plugins_url('/assets/js/knowtify.js', __FILE__), array('jquery'));

        // in javascript, object properties are accessed as ajax_object.ajax_url, ajax_object.we_value
        wp_localize_script('knowtify-admin', 'ajax_object', array('ajax_url' => admin_url('admin-ajax.php')));
    }

    /**
     * Admin AJAX response.
     */
    function signup_ajax_response() {
        $obj = new stdClass(); // Return object.
        $email = $_POST['email'];
        $email = sanitize_email($email);
        if (!is_email($email)) {
            $obj->status = "error";
            $obj->msg = "It appears that your email address is not valid.";
            $obj->field = "knowtify_signup_email";
            echo json_encode($obj);
            die();
        }
        // Process signup:
        $response = $this->create_account($email);
        $obj->status = "success";
        $obj->msg = $response;
        echo json_encode($obj);

        die();
    }

    /**
     * New reader.
     */
    function reader_ajax_response() {
        $obj = new stdClass(); // Return object.
        $name = $_POST['name'];
        $email = $_POST['email'];
        $email = sanitize_email($email);
        if (!is_email($email)) {
            $obj->status = "error";
            $obj->msg = "It appears that your email address is not valid.";
            $obj->field = "knowtify_signup_email";
            echo json_encode($obj);
            die();
        }
        // Process signup:
        $contacts = array();
        $contact = new stdClass();
        $contact->name = $name;
        $contact->email = $email;
        $contacts[] = $contact;
        $response = $this->add_contacts($contacts);
        $obj->status = "success";
        $obj->msg = $response;
        echo json_encode($obj);

        die();
    }

    /**
     * Add contacts via API.
     * @param $contacts[] $contacts Array of contact objects (name,email).
     */
    function add_contacts($contacts = array()) {

        $url = $this->apiURL . "/contacts/add";
        $args = array();
        $args['user-agent'] = 'Knowtify WP';
        $args['body'] = json_encode(array("contacts" => $contacts));
        $args['headers']['Authorization'] = 'Token token="' . $this->apiKey . '"';
        $args['sslverify'] = false;
        $call = wp_remote_post($url, $args);
        $body = json_decode($call['body']);
        /*
          ob_start();
          echo $call['body'];
          $contents = ob_get_contents();
          ob_end_clean();
          mail("allan@collinsinternet.com", "Knowtify Comment", $contents);
         * 
         */
    }

    /**
     * Create a new Knowtify account:
     * @param string $email New user's email address.
     * @return string Knowtify API response.
     */
    function create_account($email,$password) {
        $url = $this->apiURL . "/users/add";
        $args = array();
        $c = new stdClass();
        $c->source = "WordPress";
        $c->email = $email;
        $args['user-agent'] = 'Knowtify WP';
        $args['body'] = json_encode(array("user" => $c));
        $args['sslverify'] = false;
        $call = wp_remote_post($url, $args);
        /*
          ob_start();
          echo "<pre>";
          print_r($call);
          echo "</pre>";
          $contents=  ob_get_contents();
          ob_end_clean();
          mail("allan@collinsinternet.com", "knowtify signup", $contents);
         * 
         */
        $body = json_decode($call['body']);
        if (isset($body->access_token)) { // Set the API token in WP
            update_option("Knowtify_APIKEY", $body->access_token);
            $this->apiKey = $body->access_token;
            $this->update_digest();
        }

        return $body;
    }

    /**
     * Gets the number of credits remaining in users account.
     */
    function get_credits(){
        $url = $this->apiURL . "/account/credits";
        $args = array();
        $args['user-agent'] = 'Knowtify WP';
        $args['headers']['Authorization'] = 'Token token="' . $this->apiKey . '"';
        $args['sslverify'] = false;
        $call = wp_remote_post($url, $args);
        $body = json_decode($call['body']);
        return $body;
    }

    /**
     * Adds checkbox to comments form.
     */
    function comment_form() {
        if (get_option('KT_show_cb') != 1) {
            return;
        }
        include plugin_dir_path(__FILE__) . "views/comments.php";
    }

    /**
     * Checks if checkbox is submitted and checked on comment submission, if so, it adds contact.
     * @param int $commentID
     */
    function comment_subscribe($commentID) {
        if (!isset($_POST['kt_add'])) {
            return;
        }
        if ($_POST['kt_add'] != 1) {
            return;
        }
        $comment = get_comment($commentID);

        if ($comment->comment_approved == 'spam') {
            return;
        }

        $c = new stdClass();
        $c->email = $comment->comment_author_email;
        $c->name = $comment->comment_author;
        $this->add_contacts(array($c));
    }

    /**
     * Adds signup form
     */
    function signup_form($content) {
        if (get_option('KT_show_signup_form') != 1) {
            return $content;
        }
        $filename = plugin_dir_path(__FILE__) . "views/signup.php";
        if (is_file($filename)) {
            ob_start();
            include $filename;
            $content .= ob_get_contents();
            ob_end_clean();
        }
        //$content .= include plugin_dir_path(__FILE__) . "views/signup.php";
        return $content;
    }

    /**
     * Action that runs when a post is published.  It will send the post information to the Knowtify API.
     * @param int $post_id Post ID
     */
    function add_post($post_id) {
        if (( $_POST['post_status'] == 'publish' ) && ( $_POST['original_post_status'] != 'publish' )) {
            $post = get_post($post_id);
            if ($post->post_type == 'post') {  // We only need to run this on blog posts, no custom post types or pages.
                $url = $this->apiURL . "/wp/add_post";
                $params = new stdClass();
                if (has_post_thumbnail($post->ID)) {
                    //$params->post->image = get_the_post_thumbnail( $post->ID, array(75,75) );
                    $img = wp_get_attachment_image_src(get_post_thumbnail_id($post->ID), 'single-post-thumbnail');
                    $params->post->image = $img[0];
                }
                $params->post->title = $post->post_title;
                $params->post->url = get_permalink($post->ID);
                $params->post->excerpt = $post->post_excerpt;

                $args = array();
                $args['user-agent'] = 'Knowtify WP';
                $args['body'] = json_encode($params);
                $args['headers']['Authorization'] = 'Token token="' . $this->apiKey . '"';
                $args['sslverify'] = false;
                $call = wp_remote_post($url, $args);
            }
        }
    }

    function update_digest(){
        $url = $this->apiURL . "/data/edit";

        $params = new stdClass();
        $params->data->digest_subject = get_option("KT_digest_subject");
        $params->data->digest_from = get_option("KT_digest_from");
        $params->data->digest_from_email = get_option("KT_digest_from_email");
        $params->data->header_image = get_option("KT_digest_header_image");
        $params->data->heading = get_option("KT_digest_heading");
        if(get_option("KT_digest_intro")) {
            $params->data->intro = get_option("KT_digest_intro");
        }
        if(get_option("KT_digest_address")) {
            $params->data->address = get_option("KT_digest_address");
        }

        $args = array();
        $args['user-agent'] = 'Knowtify WP';
        $args['body'] = json_encode($params);
        $args['headers']['Authorization'] = 'Token token="' . $this->apiKey . '"';
        $args['sslverify'] = false;
        $call = wp_remote_post($url, $args);
    }

}
